<?php


include_once("oai2/oaidp-util.php");


/*
 * The base class of all handler factories. It basically exists here
 * to define the interface, e.g. the createHandler function. It is
 * usually subclassed and handler specific information is provided in
 * the instances of the subclass. (Alternatively a function and a set
 * of parameters could be given to the Provider)
 */
class HandlerFactory {


    /*
     * Create a MetadataRecordReader for the provider and the metadata
     * format.
     *
     * This method should be overridden in subclasses
     */
    function createHandler($invokingProvider) {
	die("HandlerFactory::createHandler: Subclass responsibility");
    }


    /*
     * Create an RecordIterator for the provider and the metadata
     * format from a file. This is used with resumption tokens.
     *
     * This method should be overridden in subclasses
     */
    function resumeFrom($invokingProvider, $fileName) {
	die("HandlerFactory::resumeFrom Subclass responsibility");
    }
}



class Provider {


    function Provider($anUrl, $providerId) {
	$this->anUrl = $anUrl;
	$this->providerId = $providerId;
	$this->theSets = array();
	$this->theFactories = array();
    }


    /*
     * Returns an array of all format prefixes that the record with
     * the identifier $identifier, is available in.
     */
    function getFormats($identifier) {
	return array_keys($this->theFactories);
    }


    function getUrl() {
	return $this->anUrl;
    }


    function addSet($setSpec, $setName = "") {
	$this->theSets[$setSpec] = $setName;
    }


    function numberOfSets() {
	return count($this->theSets);
    }


    function setsToXmlString($indent) {
	$result = "";
	foreach ($this->theSets as $setSpec => $setName) {
	    $result .= str_pad("", $indent) . "<set>\n";
	    $indent += 4;
	    $result .= str_pad("", $indent) . "<setSpec>" . xmlstrAsUtf8($setSpec) . "</setSpec>\n";
	    if ($setName) {
		$result .= str_pad("", $indent) . "<setName>" . xmlstrAsUtf8($setName) . "</setName>\n";
	    }
	    $indent -= 4;
	    $result .= str_pad("", $indent) . "</set>\n";
	}
	return $result;
    }


    function registerMetadataFormat($metadataPrefix, $handlerFactory, $schema, $nameSpace) {
	$formatInfo = array($handlerFactory, $schema, $nameSpace);
	$this->theFactories[$metadataPrefix] = $formatInfo;
    }


    function numberOfMetadataFormats() {
	return count($this->theFactories);
    }


    function getHandlerForMetadataFormat($metadataPrefix) {
	if ($metadataPrefix && array_key_exists($metadataPrefix, $this->theFactories)) {
	    $theFactory = $this->theFactories[$metadataPrefix][0];
	    return $theFactory->createHandler($this);
	}
	return NULL;
    }


    function resumeForMetadataFormatFrom($metadataPrefix, $fileName) {
	if (array_key_exists($metadataPrefix, $this->theFactories)) {
	    $theFactory = $this->theFactories[$metadataPrefix][0];
	    return $theFactory->resumeFrom($this, $fileName);
	}
	return NULL;
    }


    function metadataFormatsToXmlString($indent) {
	$result = "";
	foreach ($this->theFactories as $metadataPrefix => $formatInfo) {
	    $result .= str_pad("", $indent) . "<metadataFormat>\n";
	    $indent += 4;
	    $result .= str_pad("", $indent) . "<metadataPrefix>" . $metadataPrefix . "</metadataPrefix>\n";
	    if ($formatInfo[1]) {
		$result .= str_pad("", $indent) . "<schema>" . $formatInfo[1] . "</schema>\n";
	    }
	    if ($formatInfo[2]) {
		$result .= str_pad("", $indent) . "<metadataNamespace>" . $formatInfo[2] . "</metadataNamespace>\n";
	    }
	    $indent -= 4;
	    $result .= str_pad("", $indent) . "</metadataFormat>\n";
	}
	return $result;
    }


    function identifyAsXmlString($indent) {
	global $repositoryName;
	global $protocolVersion;
	global $adminEmail;
	global $earliestDatestamp;
	global $deletedRecord;
	global $granularity;
	global $compression;
	global $show_identifier;
	global $repositoryIdentifier;
	global $delimiter;
	global $sampleIdentifier;
	
	$result  = "";
	$result .= xmlformat($repositoryName, 'repositoryName', $indent);
	$result .= xmlformat($this->anUrl, 'baseURL', $indent);
	$result .= xmlformat($protocolVersion, 'protocolVersion', $indent);
	$result .= xmlformat($adminEmail, 'adminEmail', $indent);
	$result .= xmlformat($earliestDatestamp, 'earliestDatestamp', $indent);
	$result .= xmlformat($deletedRecord,'deletedRecord', $indent);
	$result .= xmlformat($granularity, 'granularity', $indent); 
	$result .= xmlformat($compression, 'compression', $indent);


	// A description MAY be included.
	// Use this if you choose to comply with a specific format of unique identifiers
	// for items. 
	// See http://www.openarchives.org/OAI/2.0/guidelines-oai-identifier.htm 
	// for details

	if ($show_identifier && $repositoryIdentifier && $delimiter && sampleIdentifier) {
	    $result .= 
'  <description>
   <oai-identifier xmlns="http://www.openarchives.org/OAI/2.0/oai-identifier"
                   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                   xsi:schemaLocation="http://www.openarchives.org/OAI/2.0/oai-identifier
                   http://www.openarchives.org/OAI/2.0/oai-identifier.xsd">
    <scheme>oai</scheme>
    <repositoryIdentifier>'.$repositoryIdentifier.'</repositoryIdentifier>
    <delimiter>'.$delimiter.'</delimiter>
    <sampleIdentifier>'.$sampleIdentifier.'</sampleIdentifier>
   </oai-identifier>
  </description>'."\n";
	}
	return $result;
    }
}


?>
